#include <iostream>
#include <sstream>
#include <string>
#include <unordered_map>
#include <utility>
#include <vector>

using namespace std;

unordered_map<string, long long> mem;

bool IsSauce(const string& name) { return name[0] >= 'A' && name[0] <= 'Z'; }

int FindIndexToResolve(const vector<bool>& sauce_resolved,
                       const vector<vector<pair<string, int> > >& sauce_raw,
                       const unordered_map<string, int>& sauce_name) {
  for (int i = 0; i < sauce_resolved.size(); ++i) {
    if (sauce_resolved[i]) continue;
    bool can_resolve = true;
    for (int j = 0; j < sauce_raw[i].size(); ++j) {
      string name = sauce_raw[i][j].first;
      if (!IsSauce(name)) continue;
      // cout << "name: " << name << endl;
      if (!sauce_resolved[sauce_name.at(name)]) {
        can_resolve = false;
        break;
      }
    }
    if (can_resolve) {
      return i;
    }
  }
  // cout << "Error!" << endl;
  return -1;
}

void AddTo(vector<int>& combined, const vector<int>& recipe) {
  if (combined.empty()) {
    combined = recipe;
  } else {
    for (int i = 0; i < recipe.size(); ++i) {
      combined[i] += recipe[i];
    }
  }
}

vector<int> Resolve(const vector<pair<string, int> >& raw,
                    const unordered_map<string, int>& ing_name,
                    const unordered_map<string, int>& sauce_name,
                    const vector<vector<int> >& ing_recipe,
                    const vector<vector<int> >& sauce_recipe) {
  vector<int> combined;
  for (int i = 0; i < raw.size(); ++i) {
    string name = raw[i].first;
    int units = raw[i].second;
    if (IsSauce(name)) {
      int sauce_index = sauce_name.at(name);
      for (int j = 0; j < units; ++j) {
        AddTo(combined, sauce_recipe[sauce_index]);
      }
    } else {
      int ing_index = ing_name.at(name);
      for (int j = 0; j < units; ++j) {
        AddTo(combined, ing_recipe[ing_index]);
      }
    }
  }
  return combined;
}

string RecipeToString(const vector<int>& recipe, int index) {
	string s;
	for (int i = 0; i < recipe.size(); ++i) {
		s.push_back((char)(recipe[i] + '0'));
	}
	s.push_back((char)((index / 100) % 10 + '0'));
	s.push_back((char)((index / 10) % 10 + '0'));
	s.push_back((char)(index % 10 + '0'));
	return s;
  // ostringstream oss;
  // for (int i = 0; i < recipe.size(); ++i) {
  //   oss << recipe[i] << " ";
  // }
  // oss << index;
  // return oss.str();
}

vector<int> Minus(const vector<int>& recipe_a, const vector<int>& recipe_b) {
  vector<int> recipe_c(recipe_a.size());
  for (int i = 0; i < recipe_a.size(); ++i) {
    recipe_c[i] = recipe_a[i] - recipe_b[i];
    if (recipe_c[i] < 0) return vector<int>();
  }
  return recipe_c;
}

bool AllZero(const vector<int>& recipe) {
  for (int i = 0; i < recipe.size(); ++i) {
    if (recipe[i] != 0) return false;
  }
  return true;
}

long long Compute(const vector<vector<int> >& com_recipe,
                  const vector<int>& available_com,
                  const vector<int>& target_recipe, int index) {
  const string recipe_string = RecipeToString(target_recipe, index);
  unordered_map<string, long long>::iterator iter = mem.find(recipe_string);
  if (iter != mem.end()) {
    return iter->second;
  }
  long long ans = 0;
  if (AllZero(target_recipe)) {
    ans = 1;
  } else {
    int com_index = available_com[index];
    vector<int> after_recipe = Minus(target_recipe, com_recipe[com_index]);
    if (!after_recipe.empty()) {
      ans += Compute(com_recipe, available_com, after_recipe, index);
    }
    if (index + 1 < available_com.size()) {
      ans += Compute(com_recipe, available_com, target_recipe, index + 1);
    }
  }
  // cout << "recipe: " << recipe_string << " ans: " << ans << endl;
  mem[recipe_string] = ans;
  return ans;
}

long long ComputeWays(const vector<vector<int> >& com_recipe,
                      const vector<int>& available_com,
                      const vector<int>& target_recipe) {
  // for (int i = 0; i < com_recipe.size(); ++i) {
  // 	for (int j = 0; j < com_recipe[i].size(); ++j) {
  // 		cout << com_recipe[i][j] << " ";
  // 	}
  // 	cout << endl;
  // }
  // for (int i = 0; i < available_com.size(); ++i) {
  // 	cout << available_com[i] << " ";
  // }
  // cout << endl;
  // for (int i = 0; i < target_recipe.size(); ++i) {
  // 	cout << target_recipe[i] << " ";
  // }
  // cout << endl;
  return Compute(com_recipe, available_com, target_recipe, 0);
}

int main() {
  int m;
  cin >> m;
  unordered_map<string, int> ing_name;
  for (int i = 0; i < m; ++i) {
    string name;
    cin >> name;
    ing_name[name] = i;
  }

  vector<vector<int> > ing_recipe;
  for (int i = 0; i < m; ++i) {
    vector<int> recipe(m);
    recipe[i] = 1;
    ing_recipe.push_back(recipe);
  }

  int n;
  cin >> n;
  vector<vector<pair<string, int> > > sauce_raw;
  unordered_map<string, int> sauce_name;
  for (int i = 0; i < n; ++i) {
    string name;
    cin >> name;
    sauce_name[name] = i;
    int k;
    cin >> k;
    vector<pair<string, int> > com_name_and_units;
    for (int j = 0; j < k; ++j) {
      string com_name;
      int units;
      cin >> com_name >> units;
      com_name_and_units.push_back(make_pair(com_name, units));
    }
    sauce_raw.push_back(com_name_and_units);
  }

  // cout << "Resolving sauces." << endl;

  vector<bool> sauce_resolved(n);
  vector<vector<int> > sauce_recipe(n);
  for (int i = 0; i < n; ++i) {
    int index = FindIndexToResolve(sauce_resolved, sauce_raw, sauce_name);
    // cout << "index: " << index << endl;
    sauce_resolved[index] = true;
    sauce_recipe[index] = Resolve(sauce_raw[index], ing_name, sauce_name,
                                  ing_recipe, sauce_recipe);
  }

  // cout << "Resolved sauces." << endl;

  vector<vector<int> > com_recipe;
  for (int i = 0; i < m; ++i) {
    com_recipe.push_back(ing_recipe[i]);
  }
  for (int i = 0; i < n; ++i) {
    com_recipe.push_back(sauce_recipe[i]);
  }

  int l;
  cin >> l;
  vector<int> available_com;
  for (int i = 0; i < l; ++i) {
    string name;
    cin >> name;
    int index;
    if (IsSauce(name)) {
      index = sauce_name[name] + m;
    } else {
      index = ing_name[name];
    }
    available_com.push_back(index);
  }

  vector<pair<string, int> > target_raw;
  int t;
  cin >> t;
  for (int i = 0; i < t; ++i) {
    string com_name;
    int units;
    cin >> com_name >> units;
    target_raw.push_back(make_pair(com_name, units));
  }

  vector<int> target_recipe =
      Resolve(target_raw, ing_name, sauce_name, ing_recipe, sauce_recipe);

  long long ans = ComputeWays(com_recipe, available_com, target_recipe);
  cout << ans << endl;

  return 0;
}